package com.example.sdk;

import com.alibaba.cloudapi.sdk.client.ApacheHttpClient;
import com.alibaba.cloudapi.sdk.constant.SdkConstant;
import com.alibaba.cloudapi.sdk.enums.HttpMethod;
import com.alibaba.cloudapi.sdk.enums.ParamPosition;
import com.alibaba.cloudapi.sdk.enums.Scheme;
import com.alibaba.cloudapi.sdk.model.ApiRequest;
import com.alibaba.cloudapi.sdk.model.ApiResponse;
import com.alibaba.cloudapi.sdk.model.HttpClientBuilderParams;
import com.alibaba.fastjson.JSON;

import java.nio.charset.StandardCharsets;
import java.util.Map;

import com.example.sdk.common.GatewayResponse;
import lombok.Getter;

/**
 * 封装后的通用网关调用客户端
 */
public class GatewayApiClient extends ApacheHttpClient {

    @Getter
    private static final GatewayApiClient instance = new GatewayApiClient();

    private GatewayApiClient() {
    }

    /**
     * 初始化客户端，供外部调用
     */
    @Override
    public void init(HttpClientBuilderParams httpClientBuilderParams) {
        httpClientBuilderParams.setScheme(Scheme.HTTPS);
        super.init(httpClientBuilderParams);
    }

    /**
     * 无自定义 Header 的快捷方法
     */
    public <T> GatewayResponse<T> invoke(String path, Object params, Class<T> responseType) {
        return invoke(path, params, responseType, null);
    }

    /**
     * 核心方法 - 发起请求并返回网关通用响应
     *
     * @param path         请求路径 (例如 "/api/v1/user/create")
     * @param params       请求体，对应具体业务 DTO
     * @param responseType data字段对应的Class类型，用于反序列化
     * @param headers      可选的自定义头
     */
    public <T> GatewayResponse<T> invoke(
        String path,
        Object params,
        Class<T> responseType,
        Map<String, String> headers
    ) {
        // 1. 构造请求体
        byte[] body = null;
        try {
            if (params != null) {
                body = JSON.toJSONString(params).getBytes(StandardCharsets.UTF_8);
            }
        } catch (Exception e) {
            throw new RuntimeException("Failed to serialize request body", e);
        }

        ApiRequest request = new ApiRequest(HttpMethod.POST_BODY, path, body);
        request.addHeader("Content-Type", "application/json");

        // 2. 如果有自定义Header，则加入请求
        if (headers != null && !headers.isEmpty()) {
            for (Map.Entry<String, String> entry : headers.entrySet()) {
                request.addParam(entry.getKey(), entry.getValue(), ParamPosition.HEAD, false);
            }
        }

        // 3. 发送请求
        ApiResponse apiResponse = sendSyncRequest(request);

        // 4. 将返回的byte[] 转换成网关层通用响应
        if (apiResponse.getCode() >= 200 && apiResponse.getCode() < 300) {
            // HTTP响应成功
            String responseJson = new String(apiResponse.getBody(), SdkConstant.CLOUDAPI_ENCODING);
            // 这里使用了 fastjson 提供的泛型反序列化方式
            return JSON.parseObject(responseJson,
                // 这里使用了 fastjson 提供的泛型反序列化方式
                new com.alibaba.fastjson.TypeReference<GatewayResponse<T>>(responseType) {}
            );
        } else {
            // HTTP层就失败了，可以根据需要封装成错误返回
            GatewayResponse<T> errorResponse = new GatewayResponse<>();
            errorResponse.setStatus(20);
            errorResponse.setErrCode("HTTP_ERROR_" + apiResponse.getCode());
            errorResponse.setErrMessage("HTTP error occurred");
            return errorResponse;
        }
    }
}